#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Tech - carte
# 
# Faire une carte, c'est toujours compliqué. C'est simple jusqu'à ce qu'on s'aperçoive qu'on doit récupérer la description des zones administratives d'un pays, fournies parfois dans des coordonnées autres que longitude et latitude. Quelques modules utiles :
# 
# * [cartopy](https://scitools.org.uk/cartopy/docs/latest/) : surcouche de matplotlib pour faire des dessins avec des coordonnées géographiques
# * [bokeh](https://docs.bokeh.org/) : pour tracer des cartes interactives
# * [pyproj](https://github.com/pyproj4/pyproj) : conversion entre systèmes de coordonnées
# * [shapely](https://shapely.readthedocs.io/en/stable/manual.html) : manipuler des polygones géographiques (union, intersection, ...)
# * [pyshp](https://github.com/GeospatialPython/pyshp) : lire ou écrire des polygones géographiques
# * [geopandas](https://geopandas.org/) : manipulation de dataframe avec des coordonnées géographiques
# 
# Quelques notebooks intéressants :
# * [Tracer une carte en Python avec bokeh](http://195-154-200-30.rev.poneytelecom.eu/app/papierstat/helpsphinx/notebooks/enedis_cartes_bokeh.html)
# * [Tracer une carte en Python](http://195-154-200-30.rev.poneytelecom.eu/app/papierstat/helpsphinx/notebooks/enedis_cartes.html)
# * [Données carroyées et OpenStreetMap](http://195-154-200-30.rev.poneytelecom.eu/app/papierstat/helpsphinx/notebooks/carte_carreau.html)
# * [Carte de France avec les départements](http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx/notebooks/td1a_cenonce_session_12_carte.html)
# * [Carte de France avec les départements (2)](http://www.xavierdupre.fr/app/actuariat_python/helpsphinx/notebooks/seance6_graphes_correction.html)

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Exposé
# 
# On télécharge des [données hospitalières par départements](https://www.data.gouv.fr/fr/datasets/donnees-hospitalieres-relatives-a-lepidemie-de-covid-19/).

# ### Données COVID

# In[3]:


# https://www.data.gouv.fr/fr/datasets/donnees-hospitalieres-relatives-a-lepidemie-de-covid-19/
from pandas import read_csv
url = "https://www.data.gouv.fr/fr/datasets/r/63352e38-d353-4b54-bfd1-f1b3ee1cabd7"
covid = read_csv(url, sep=";")
covid.tail()


# In[4]:


last_day = covid.loc[covid.index[-1], "jour"]
last_day


# In[5]:


last_data = covid[covid.jour == last_day].groupby("dep").sum(numeric_only=True)
last_data.shape


# In[6]:


last_data.describe()


# In[7]:


last_data.head()


# In[8]:


last_data.tail()


# ### Données départements

# On récupère ensuite la définition [géographique des départements](https://www.data.gouv.fr/en/datasets/contours-geographiques-des-departements/).

# In[9]:


import geopandas
# dernier lien de la page (format shapefiles)
url = "https://www.data.gouv.fr/fr/datasets/r/90b9341a-e1f7-4d75-a73c-bbc010c7feeb"
geo = geopandas.read_file(url)
geo.tail()


# Il faudrait aussi fusionner avec la population de chaque département. Ce sera pour une autre fois. 

# ### Carte

# In[10]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 1, figsize=(5, 4))
geo.plot(ax=ax, color='white', edgecolor='black');


# On enlève tous les départements à trois chiffres. Les départements outre-mer sont présents ou non selon le jeu de données téléchargé.

# In[11]:


codes = [_ for _ in set(geo.code) if len(_) < 3]
metropole = geo[geo.code.isin(codes)]
metropole.tail()


# In[12]:


fig, ax = plt.subplots(1, 1, figsize=(5, 4))
metropole.plot(ax=ax, color='white', edgecolor='black')
ax.set_title("%s départements" % metropole.shape[0]);


# ### Carte COVID

# In[13]:


merged = last_data.reset_index(drop=False).merge(metropole, left_on="dep", right_on="code")
merged.shape


# In[14]:


merged.tail()


# In[15]:


fig, ax = plt.subplots(1, 1, figsize=(5, 4))
merged.hist('rea', bins=20, ax=ax)
ax.set_title("Distribution rea");


# Les régions les plus peuplées ont sans doute la plus grande capacité hospitalière. Il faudrait diviser par cette capacité pour avoir une carte qui ait un peu plus de sens. Comme l'idée est ici de simplement tracer la carte, on ne calculera pas de ratio.

# In[16]:


merged.sort_values('rea').tail()


# In[17]:


geomerged = geopandas.GeoDataFrame(merged)


# In[18]:


from mpl_toolkits.axes_grid1 import make_axes_locatable

fig, ax = plt.subplots(1, 1)

# ligne à ajouter pour avoir une légende ajustée à la taille du graphe
cax = make_axes_locatable(ax).append_axes("right", size="5%", pad=0.1)

geomerged.plot(column="rea", ax=ax, edgecolor='black', legend=True, cax=cax)
ax.set_title("Réanimations pour les %d départements" % metropole.shape[0]);


# La création de carte a toujours été plus ou moins compliqué. Les premiers notebooks que j'ai créés sur le sujet étaient beaucoup plus complexe. *geopandas* a simplifié les choses. Son développement a commencé entre [2013](https://github.com/geopandas/geopandas/graphs/contributors) et a bien évolué depuis. Et j'ai dû passer quelques heures à récupérer les contours des départements il y a cinq ans.

# On peut également récupérer la capacité maximale de chaque département en regardant sur le passé.

# In[19]:


capacite = covid.groupby(["jour", "dep"]).sum().groupby("dep").max()
capacite.head()


# In[20]:


capa_merged = merged.merge(capacite, left_on="dep", right_on="dep")
capa_merged["occupation"] = capa_merged["rea_x"] / capa_merged["rea_y"]
capa_merged.head(n=2).T


# In[21]:


geocapa = geopandas.GeoDataFrame(capa_merged)


# In[22]:


fig, ax = plt.subplots(1, 1)

# ligne à ajouter pour avoir une légende ajustée à la taille du graphe
cax = make_axes_locatable(ax).append_axes("right", size="5%", pad=0.1)

geocapa.plot(column="occupation", ax=ax, edgecolor='black', legend=True, cax=cax)
ax.set_title("Occupations en réanimations pour les %d départements" % metropole.shape[0]);


# 

# In[23]: